@@ -1,6 +1,9 @@
 from .lit import (
     start_effect,
     start_preset,
+    stop,
+    forward,
+    back,
     query,
     dev_command,
     get_effects,
