@@ -2,7 +2,7 @@ import random
 
 name = "Multi-Chase"
 
-start_string = name + " started!"
+start_message = name + " started!"
 
 description = (
     "The string is sequentially covered by random colors with multiple at the same time"
@@ -17,11 +17,6 @@ def create_heads(lights, args):
 
 
 schema = {
-    "speed": {
-        "value": {"type": "number", "min": 0, "max": 100, "default": 50},
-        "user_input": True,
-        "required": False,
-    },
     "number": {
         "value": {
             "type": "number",  # TODO integer type (or step)
