@@ -1,16 +1,10 @@
 name = "Slide"
 
-start_string = name + " started!"
+start_message = name + " started!"
 
 description = "The full color spectrum shifts along the strand."
 
-schema = {
-    "speed": {
-        "value": {"type": "number", "min": 0, "max": 100, "default": 50},
-        "user_input": True,
-        "required": False,
-    }
-}
+schema = {}
 
 
 def update(lights, step, state):
