@@ -5,7 +5,7 @@ logger = logging.getLogger(__name__)
 
 name = "Twinkle"
 
-start_string = name + " started!"
+start_message = name + " started!"
 
 description = "Like a little star"
 
@@ -15,11 +15,6 @@ def setup_start_durations(lights, args):
 
 
 schema = {
-    "speed": {
-        "value": {"type": "number", "min": 1, "max": 100, "default": 50},
-        "user_input": True,
-        "required": False,
-    },
     "color": {
         "value": {"type": "color", "default": (255, 255, 255)},
         "user_input": True,
